
loglk <- function(mu) {x <<- x0;  sigma <<- sigma;       #  negative log likelihood  -log lk  
            (-log(sigma)+((x-mu)^2)/(2*sigma^2))
}

dloglk <- function(mu) { x <<- x0; sigma <<- sigma;      # derivative of negative log likelihood  -log lk
              -(x - mu)/(sigma^2)
}

set.seed(5315)                  # setting the seed of random values
x0 <- 10                        # data(sample size = 1)
sigma  <-  1                    # std


mu0 <- 1                        # initial value
N <- 100000                     # number of random values to be generated

mu <- numeric(N)                # memory for storing the random values of mu
U <- numeric(N)                 # memory for potential energy
H <- numeric(N)                 # memory for hamiltonian


eps = 0.1                       # stepsize of leapfrog
L <- 15                          # number of steps of leapfrog
accepted <- 0                   # number of accepted candidates

for (i in 1:N) {
  if (i==1)
  {
    mu[i] <- mu0
  }else{
    mu[i] <- mu[i-1]                 # initial value for mu
  }
  pm <- rnorm(1)                     # new pm
  Hini <- pm^2 / 2 + loglk(mu[i])    # Hamiltonian of new pm and mu
  U[i] <- loglk(mu[i])               # i-th value of potential E

  for (k in 2:L) {                   # leapfrog method 
    pm = pm - eps * dloglk(mu[i])/2      
    mu[i] = mu[i] + eps * pm
    pm = pm - eps * dloglk(mu[i])/2
  }

  H[i] <- pm^2 / 2 + loglk(mu[i])    # Hamiltonian after Euler's method

  if (log(runif(1)) < Hini - H[i]){  # MH method
    accepted <- accepted + 1
  }else{
    if(i>1){
      mu[i] <- mu[i-1] 
      U[i] <- U[i-1]
      H[i] <- H[i-1]
    }
  }
}

par(mfrow=c(3,1))
plot(mu,type="l",cex.axis=1.5,xlab="i",ylab="",cex.lab=1.5)
plot(H,type="l",cex.axis=1.5,xlab="i",ylab="H",cex.lab=1.5)
plot(U,type="l",cex.axis=1.5,xlab="i",ylab="U",cex.lab=1.5)

par(mfrow=c(1,1))
hist(mu, col="white", xlab="mu",ylab="",main="",freq=FALSE, breaks=100)
x<-mu
curve(dnorm(x, x0, sigma), add=TRUE, col="red")

accepted                  #number of accepted candidates

